@@ -27,6 +27,7 @@
 
 #include <QtCore/QDir>
 #include <QProcess>
+#include <QTemporaryFile>
 
 #include <kconfig.h>
 #include <kde_file.h>
@@ -34,7 +35,9 @@
 #include <kstandarddirs.h>
 #include <kconfiggroup.h>
 #include <kcomponentdata.h>
-#include "ktemporaryfile.h"
+
+#include <stdlib.h>
+#include <errno.h>
 
 class KSaveFile::Private
 {
@@ -45,7 +48,6 @@ public:
     QFile::FileError error;
     QString errorString;
     bool wasFinalized;
-    FILE *stream;
     KComponentData componentData;
 
     Private(const KComponentData &c)
@@ -53,7 +55,6 @@ public:
     {
         error = QFile::NoError;
         wasFinalized = false;
-        stream = 0;
     }
 };
 
@@ -76,7 +77,7 @@ KSaveFile::~KSaveFile()
     delete d;
 }
 
-bool KSaveFile::open()
+bool KSaveFile::open(OpenMode flags)
 {
     if ( d->realFileName.isNull() ) {
         d->error=QFile::OpenError;
@@ -85,8 +86,10 @@ bool KSaveFile::open()
     }
 
     if ( !d->tempFileName.isNull() ) {
+#if 0 // do not set an error here, this open() fails, but the file itself is without errors
         d->error=QFile::OpenError;
         d->errorString=i18n("Already opened.");
+#endif
         return false;
     }
 
@@ -100,7 +103,7 @@ bool KSaveFile::open()
     }
 
     //Create our temporary file
-    KTemporaryFile tempFile(d->componentData);
+    QTemporaryFile tempFile;
     tempFile.setAutoRemove(false);
     tempFile.setFileTemplate(d->realFileName + "XXXXXX.new");
     if (!tempFile.open()) {
@@ -111,27 +114,22 @@ bool KSaveFile::open()
     
     // if we're overwriting an existing file, ensure temp file's
     // permissions are the same as existing file so the existing
-    // file's permissions are preserved; but only if we are the
-    // same owner and group.
+    // file's permissions are preserved. this will succeed only if we
+    // are the same owner and group - or allmighty root.
     QFileInfo fi ( d->realFileName );
-    if (fi.ownerId() == getuid()) {
-        bool changePermission = true;
-        if (fi.groupId() != getgid()) {
-            //Qt apparently has no way to change owner/group of file :(
-            if (fchown(tempFile.handle(), (uid_t) -1, fi.groupId()) != 0) {
-                // Use standard permission if we can't set the group
-                changePermission = false;
-             }
-        }
-        
-        if (changePermission) {
+    if (fi.exists()) {
+        //Qt apparently has no way to change owner/group of file :(
+        if (!fchown(tempFile.handle(), fi.ownerId(), fi.groupId()))
             tempFile.setPermissions(fi.permissions());
-        }
+    }
+    else {
+        mode_t umsk = KGlobal::umask();
+        fchmod(tempFile.handle(), 0666&(~umsk));
     }
 
     //Open oursleves with the temporary file
     QFile::setFileName(tempFile.fileName());
-    if ( !QFile::open(QIODevice::ReadWrite) ) {
+    if (!QFile::open(flags)) {
         tempFile.setAutoRemove(true);
         return false;
     }
@@ -153,6 +151,10 @@ void KSaveFile::setFileName(const QString &filename)
 
     // follow symbolic link, if any
     d->realFileName = KStandardDirs::realFilePath( d->realFileName );
+
+#ifdef Q_WS_WIN
+    d->realFileName.replace(QString::fromLatin1("//"), QString::fromLatin1("/"));
+#endif
     return;
 }
 
@@ -181,33 +183,53 @@ QString KSaveFile::fileName() const
 
 void KSaveFile::abort()
 {
-    if ( d->stream ) {
-        fclose(d->stream);
-        d->stream = 0;
-    }
-    
     close();
     QFile::remove(d->tempFileName); //non-static QFile::remove() does not work. 
     d->wasFinalized = true;
 }
 
+#ifdef HAVE_FDATASYNC
+#  define FDATASYNC fdatasync
+#else
+#  define FDATASYNC fsync
+#endif
+
 bool KSaveFile::finalize()
 {
     bool success = false;
 
     if ( !d->wasFinalized ) {
-        if ( d->stream ) {
-            fclose(d->stream);
-            d->stream = 0;
-        }
-        close();
 
+#ifdef Q_OS_UNIX
+        static int extraSync = -1;
+        if (extraSync < 0)
+            extraSync = getenv("KDE_EXTRA_FSYNC") != 0 ? 1 : 0;
+        if (extraSync) {
+            if (flush()) {
+                forever {
+                    if (!FDATASYNC(handle()))
+                        break;
+                    if (errno != EINTR) {
+                        d->error = QFile::WriteError;
+                        d->errorString = i18n("Synchronization to disk failed");
+                        break;
+                    }
+                }
+            }
+        }
+#endif
+
+        close();
+        
+        if( error() != NoError ) {
+            QFile::remove(d->tempFileName);
+        }
         //Qt does not allow us to atomically overwrite an existing file,
         //so if the target file already exists, there is no way to change it
         //to the temp file without creating a small race condition. So we use
         //the standard rename call instead, which will do the copy without the
         //race condition.
-        if (0 == KDE_rename(QFile::encodeName(d->tempFileName),
+        else if (0 == KDE_rename(QFile::encodeName(d->tempFileName),
                       QFile::encodeName(d->realFileName))) {
             d->error=QFile::NoError;
             d->errorString.clear();
@@ -215,7 +237,7 @@ bool KSaveFile::finalize()
         } else {
             d->error=QFile::OpenError;
             d->errorString=i18n("Error during rename.");
-            QFile::remove();
+            QFile::remove(d->tempFileName);
         }
 
         d->wasFinalized = true;
@@ -224,6 +246,8 @@ bool KSaveFile::finalize()
     return success;
 }
 
+#undef FDATASYNC
+
 bool KSaveFile::backupFile( const QString& qFilename, const QString& backupDir )
 {
     // get backup type from config, by default use "simple"
@@ -256,6 +280,7 @@ bool KSaveFile::simpleBackupFile( const QString& qFilename,
     }
 
 //    kDebug(180) << "KSaveFile copying " << qFilename << " to " << backupFileName;
+    QFile::remove(backupFileName);
     return QFile::copy(qFilename, backupFileName);
 }
 
