@@ -1,6 +1,7 @@
 /* This file is part of the KDE libraries
    Copyright (C) 1999 Sirtaj Singh Kanq <taj@kde.org>
    Copyright (C) 2007 Matthias Kretz <kretz@kde.org>
+   Copyright (C) 2009 Olivier Goffart <ogoffart@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -28,6 +29,12 @@
 #include "kglobal.h"
 #include "kglobal_p.h"
 
+#include <config.h>
+
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
 #include <QtCore/QList>
 #include <QtCore/QSet>
 
@@ -40,6 +47,7 @@
 #include <QtCore/QCoreApplication>
 #include <QtCore/QTextCodec>
 #include "kcmdlineargs.h"
+#include <unistd.h> // umask
 
 #ifndef NDEBUG
 #define MYASSERT(x) if (!x) \
@@ -50,7 +58,12 @@
 #define MYASSERT(x) /* nope */
 #endif
 
+// ~KConfig needs qrand(). qrand() depends on a Q_GLOBAL_STATIC. With this Q_CONSTRUCTOR_FUNCTION we
+// try to make qrand() live longer than any KConfig object.
+Q_CONSTRUCTOR_FUNCTION(qrand)
+
 typedef QSet<QString> KStringDict;
+mode_t s_umsk;
 
 class KGlobalPrivate
 {
@@ -60,10 +73,10 @@ class KGlobalPrivate
             locale(0),
             charsets(0)
         {
-            // make sure all Qt/KDE global/local statics that we need are created here, that way
-            // we may use them in the dtor as well
-            qrand();
-            KStandardDirs::installPath(0);
+            // the umask is read here before any threads are created to avoid race conditions
+            mode_t tmp = 0;
+            s_umsk = umask(tmp);
+            umask(s_umsk);
         }
 
         inline ~KGlobalPrivate()
@@ -81,31 +94,44 @@ class KGlobalPrivate
         KStringDict *stringDict;
         KLocale *locale;
         KCharsets *charsets;
+
+        /**
+         * This componenent may be used instead of the main component for application
+         * That doesn't have a main componennt such as pure Qt application.
+         */
+        static KComponentData initFakeComponent()
+        {
+            QString name = QCoreApplication::applicationName();
+            if(name.isEmpty())
+                name = qAppName();
+            if(name.isEmpty())
+                name = QString::fromLatin1("kde");
+            return KComponentData(name.toLatin1(), name.toLatin1(),
+                                  KComponentData::SkipMainComponentRegistration);
+        }
 };
 
 K_GLOBAL_STATIC(KGlobalPrivate, globalData)
+K_GLOBAL_STATIC_WITH_ARGS(KComponentData, fakeComponent, (KGlobalPrivate::initFakeComponent()))
 
 #define PRIVATE_DATA KGlobalPrivate *d = globalData
 
 KStandardDirs *KGlobal::dirs()
 {
     PRIVATE_DATA;
-    MYASSERT(d->mainComponent.isValid());
-    return d->mainComponent.dirs();
+    return d->mainComponent.isValid() ? d->mainComponent.dirs() : fakeComponent->dirs();
 }
 
 KSharedConfig::Ptr KGlobal::config()
 {
     PRIVATE_DATA;
-    MYASSERT(d->mainComponent.isValid());
-    return d->mainComponent.config();
+    return d->mainComponent.isValid() ? d->mainComponent.config() : fakeComponent->config();
 }
 
 const KComponentData &KGlobal::mainComponent()
 {
     PRIVATE_DATA;
-    MYASSERT(d->mainComponent.isValid());
-    return d->mainComponent;
+    return d->mainComponent.isValid() ? d->mainComponent : *fakeComponent;
 }
 
 bool KGlobal::hasMainComponent()
@@ -125,14 +151,10 @@ KLocale *KGlobal::locale()
             return 0;
         }
 
-        // will set d->locale if it works - otherwise 0 is returned
         d->locale = new KLocale(d->mainComponent.catalogName());
         QTextCodec::setCodecForLocale(d->locale->codecForEncoding());
-        if (d->mainComponent.aboutData()) {
-            d->mainComponent.aboutData()->translateInternalProgramName();
-        }
+        d->mainComponent.aboutData()->translateInternalProgramName();
     }
-
     return d->locale;
 }
 
@@ -155,6 +177,12 @@ KCharsets *KGlobal::charsets()
     return d->charsets;
 }
 
+mode_t KGlobal::umask()
+{
+    // Don't use PRIVATE_DATA here. This is called by ~KGlobalPrivate -> ~KConfig -> sync -> KSaveFile, so there's no KGlobalPrivate anymore.
+    return s_umsk;
+}
+
 KComponentData KGlobal::activeComponent()
 {
     PRIVATE_DATA;
@@ -181,9 +209,12 @@ void KGlobal::newComponentData(const KComponentData &c)
     KGlobal::setActiveComponent(c);
 }
 
-void KGlobal::setLocale(KLocale *locale)
+void KGlobal::setLocale(KLocale *locale, CopyCatalogs copy)
 {
     PRIVATE_DATA;
+    if (copy == DoCopyCatalogs && d->locale)
+        d->locale->copyCatalogsTo(locale);
+    delete d->locale;
     d->locale = locale;
 }
 
@@ -223,7 +254,7 @@ QString KGlobal::caption()
         return args->getOption("caption");
     } else {
         // We have some about data ?
-        if (d->mainComponent.aboutData()) {
+        if (d->mainComponent.isValid() && d->mainComponent.aboutData()) {
             return d->mainComponent.aboutData()->programName();
         } else {
             // Last resort : application name
@@ -234,11 +265,14 @@ QString KGlobal::caption()
 
 /**
  * This counter indicates when to quit the application.
- * It starts at 1, is decremented in KMainWindow when the last window is closed, but
- * is incremented by operations that should outlive the last window closed
- * (e.g. a file copy for a file manager, or 'compacting folders on exit' for a mail client).
+ * It starts at 0, is incremented by KMainWindow, systray icons, running jobs, etc.
+ * and decremented again when those things are destroyed.
+ * This mechanism allows dialogs and jobs to outlive the last window closed
+ * e.g. a file copy for a file manager, or 'compacting folders on exit' for a mail client,
+ * the job progress widget with "keep open" checked, etc.
  */
-static int s_refCount = 1;
+static int s_refCount = 0;
+static bool s_allowQuit = false;
 
 void KGlobal::ref()
 {
@@ -250,9 +284,14 @@ void KGlobal::deref()
 {
     --s_refCount;
     //kDebug() << "KGlobal::deref() : refCount = " << s_refCount;
-    if (s_refCount <= 0) {
+    if (s_refCount <= 0 && s_allowQuit) {
         QCoreApplication::instance()->quit();
     }
 }
 
+void KGlobal::setAllowQuit(bool allowQuit)
+{
+    s_allowQuit = allowQuit;
+}
+
 #undef PRIVATE_DATA
