@@ -77,14 +77,15 @@ public:
     QTextCodec *m_codec;
     QTextDecoder *m_decoder; // utf16
     QTextCodec *m_defaultCodec;
+    QByteArray  m_storeDecoderName;
 
     KEncodingDetector::EncodingChoiceSource m_source;
     KEncodingDetector::AutoDetectScript m_autoDetectLanguage;
 
-    bool m_visualRTL;
-    bool m_seenBody;
-    bool m_writtingHappened;
-    bool m_analyzeCalled; //for decode()
+    bool m_visualRTL : 1;
+    bool m_seenBody : 1;
+    bool m_writtingHappened : 1;
+    bool m_analyzeCalled : 1; //for decode()
     int m_multiByte;
 
     QByteArray m_bufferForDefferedEncDetection;
@@ -121,6 +122,12 @@ public:
     {
         delete m_decoder;
     }
+
+    // Returns true if the encoding was explicitly specified someplace.
+    bool isExplicitlySpecifiedEncoding()
+    {
+        return m_source != KEncodingDetector::DefaultEncoding && m_source != KEncodingDetector::AutoDetectedEncoding;
+    }
 };
 
 
@@ -670,7 +677,8 @@ KEncodingDetector::EncodingChoiceSource KEncodingDetector::encodingChoiceSource(
 
 const char* KEncodingDetector::encoding() const
 {
-    return d->m_codec->name().data();
+    d->m_storeDecoderName = d->m_codec->name();
+    return d->m_storeDecoderName.constData();
 }
 
 bool KEncodingDetector::visuallyOrdered() const
@@ -714,7 +722,12 @@ bool KEncodingDetector::setEncoding(const char *_encoding, EncodingChoiceSource
     }
 
     if (d->m_codec->mibEnum()==codec->mibEnum())
+    {
+        // We already have the codec, but we still want to re-set the type,
+        // as we may have overwritten a default with a detected
+        d->m_source = type;
         return true;
+    }
 
     if ((type==EncodingFromMetaTag || type==EncodingFromXMLHeader) && is16Bit(codec))
     {
@@ -784,13 +797,9 @@ QString KEncodingDetector::decodeWithBuffering(const char *data, int len)
     {
         if (d->m_bufferForDefferedEncDetection.isEmpty())
         {
-            if (analyze(data,len)
-                && (d->m_seenBody
-                    || !(d->m_source==AutoDetectedEncoding
-                        ||d->m_source==DefaultEncoding
-                        )
-                   )
-               )//dontWannaSeeHead()
+            // If encoding detection produced something, and we either got to the body or
+            // actually saw the encoding explicitly, we're done.
+            if (analyze(data,len) && (d->m_seenBody || d->isExplicitlySpecifiedEncoding()))
             {
 #ifdef DECODE_DEBUG
                 kWarning() << "KEncodingDetector: m_writtingHappened first time "<< d->m_codec->name();
@@ -810,14 +819,11 @@ QString KEncodingDetector::decodeWithBuffering(const char *data, int len)
         else
         {
             d->m_bufferForDefferedEncDetection+=data;
-            if ( (analyze(data,len)
-                  && (d->m_seenBody
-                     || !(d->m_source==AutoDetectedEncoding
-                        ||d->m_source==DefaultEncoding
-                         )
-                     )
-                 ) || d->m_bufferForDefferedEncDetection.length()>MAX_BUFFER
-               )//dontWannaSeeHead()
+            // As above, but also limit the buffer size. We must use the entire buffer here,
+            // since the boundaries might split the meta tag, etc.
+            bool detected = analyze(d->m_bufferForDefferedEncDetection.constData(), d->m_bufferForDefferedEncDetection.length());
+            if ((detected && (d->m_seenBody || d->isExplicitlySpecifiedEncoding())) ||
+                 d->m_bufferForDefferedEncDetection.length() > MAX_BUFFER)
             {
                 d->m_writtingHappened=true;
                 d->m_bufferForDefferedEncDetection.replace('\0',' ');
@@ -902,7 +908,9 @@ bool KEncodingDetector::analyze(const char *data, int len)
             //enc = d->m_codec->name();
             delete d->m_decoder;
             d->m_decoder = d->m_codec->makeDecoder();
+#ifdef DECODE_DEBUG
             kWarning() << "Detection by BOM";
+#endif
             if (is16Bit(d->m_codec) && c2==0x00)
             {
                 // utf16LE, we need to put the decoder in LE mode
@@ -925,6 +933,10 @@ bool KEncodingDetector::analyze(const char *data, int len)
         return true;
     }
 
+    // HTTP header takes precedence over meta-type stuff
+    if (d->m_source==EncodingFromHTTPHeader)
+        return true;
+
     if (!d->m_seenBody)
     {
         // we still don't have an encoding, and are in the head
@@ -969,8 +981,8 @@ bool KEncodingDetector::analyze(const char *data, int len)
 
             //look for <meta>, stop if we reach <body>
             while (
-                        !((*ptr >= 'a') && (*ptr <= 'z') ||
-                        (*ptr >= 'A') && (*ptr <= 'Z'))
+                        !(((*ptr >= 'a') && (*ptr <= 'z')) ||
+                        ((*ptr >= 'A') && (*ptr <= 'Z')))
                         && ptr < pEnd
                 )
                 ++ptr;
@@ -981,9 +993,9 @@ bool KEncodingDetector::analyze(const char *data, int len)
             if (pEnd<max)
                 max=pEnd;
             while (
-                        ((*ptr >= 'a') && (*ptr <= 'z') ||
-                        (*ptr >= 'A') && (*ptr <= 'Z') ||
-                        (*ptr >= '0') && (*ptr <= '9'))
+                        (((*ptr >= 'a') && (*ptr <= 'z')) ||
+                        ((*ptr >= 'A') && (*ptr <= 'Z')) ||
+                        ((*ptr >= '0') && (*ptr <= '9')))
                         && ptr < max
                 )
             {
@@ -1011,9 +1023,18 @@ bool KEncodingDetector::analyze(const char *data, int len)
                 if( (pos = str.indexOf("=", pos)) == -1)
                     continue;
 
+                // skip '='
+                ++pos;
+
                 // skip whitespace before encoding itself
                 while (pos < (int)str.length() && str[pos] <= ' ')
                     ++pos;
+                
+                // there may also be an opening quote, if this is a charset= and not 
+                // a http-equiv.
+                if (pos < (int)str.length() && str[pos] == '"')
+                    ++pos;
+
                 if ( pos == (int)str.length())
                     continue;
 
@@ -1036,14 +1057,9 @@ bool KEncodingDetector::analyze(const char *data, int len)
         }
     }
 
-    if (d->m_source==EncodingFromHTTPHeader)
-        return true;
-
     if (len<20)
-    {
-        setEncoding("",DefaultEncoding);
         return false;
-    }
+
 #ifdef DECODE_DEBUG
     kDebug( 6005 ) << "KEncodingDetector: using heuristics (" << strlen(data) << ")";
 #endif
@@ -1098,10 +1114,9 @@ bool KEncodingDetector::analyze(const char *data, int len)
             // huh. somethings broken in this code ### FIXME
             //enc = 0; //Reset invalid codec we tried, so we get back to latin1 fallback.
             break;
-        }
+    }
 
-        setEncoding("",DefaultEncoding);
-        return true;
+    return true;
 }
 
 
