@@ -26,19 +26,33 @@
 
 #include "kjs/global.h"
 
+#include <wtf/AlwaysInline.h>
 #include <wtf/FastMalloc.h>
 #include <wtf/RefPtr.h>
 #include <wtf/PassRefPtr.h>
+#include <wtf/Vector.h>
 
+#include <assert.h>
+#include "collector.h"
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
 
+/* On some ARM platforms GCC won't pack structures by default so sizeof(UChar)
+   will end up being != 2 which causes crashes since the code depends on that. */
+#if COMPILER(GCC) && PLATFORM(FORCE_PACK)
+#define PACK_STRUCT __attribute__((packed))
+#else
+#define PACK_STRUCT
+#endif
+
 /**
  * @internal
  */
 namespace DOM {
   class DOMString;
+}
+namespace khtml {
   class AtomicString;
 }
 class QString;
@@ -46,7 +60,6 @@ class QConstString;
 
 namespace KJS {
 
-  class UCharReference;
   class UString;
 
   /**
@@ -58,7 +71,7 @@ namespace KJS {
    */
   struct KJS_EXPORT UChar {
     /**
-     * Construct a character with uninitialized value.    
+     * Construct a character with uninitialized value.
      */
     UChar();
     /**
@@ -74,22 +87,23 @@ namespace KJS {
     UChar(char u);
     UChar(unsigned char u);
     UChar(unsigned short u);
-    UChar(const UCharReference &c);
     /**
      * @return The higher byte of the character.
      */
-    unsigned char high() const { return uc >> 8; }
+    unsigned char high() const { return static_cast<unsigned char>(uc >> 8); }
+
     /**
      * @return The lower byte of the character.
      */
-    unsigned char low() const { return (unsigned char)uc; }
+    unsigned char low() const { return static_cast<unsigned char>(uc); }
+
     /**
      * @return the 16 bit Unicode value of the character
      */
     unsigned short unicode() const { return uc; }
 
     unsigned short uc;
-  };
+  } PACK_STRUCT;
 
   inline UChar::UChar() { }
   inline UChar::UChar(unsigned char h , unsigned char l) : uc(h << 8 | l) { }
@@ -97,55 +111,6 @@ namespace KJS {
   inline UChar::UChar(unsigned char u) : uc(u) { }
   inline UChar::UChar(unsigned short u) : uc(u) { }
 
-  /**
-   * @short Dynamic reference to a string character.
-   *
-   * UCharReference is the dynamic counterpart of UChar. It's used when
-   * characters retrieved via index from a UString are used in an
-   * assignment expression (and therefore can't be treated as being const):
-   * \code
-   * UString s("hello world");
-   * s[0] = 'H';
-   * \endcode
-   *
-   * If that sounds confusing your best bet is to simply forget about the
-   * existence of this class and treat is as being identical to UChar.
-   */
-  class KJS_EXPORT UCharReference {
-    friend class UString;
-    UCharReference(UString *s, unsigned int off) : str(s), offset(off) { }
-  public:
-    /**
-     * Set the referenced character to c.
-     */
-    UCharReference& operator=(UChar c);
-    /**
-     * Same operator as above except the argument that it takes.
-     */
-    UCharReference& operator=(char c) { return operator=(UChar(c)); }
-    /**
-     * @return Unicode value.
-     */
-    unsigned short unicode() const { return ref().uc; }
-    /**
-     * @return Lower byte.
-     */
-    unsigned char low() const { return static_cast<unsigned char>(ref().uc); }
-    /**
-     * @return Higher byte.
-     */
-    unsigned char high() const { return ref().uc >> 8; }
-  private:
-    // not implemented, can only be constructed from UString
-    UCharReference();
-
-    UChar& ref() const;
-    UString *str;
-    int offset;
-  };
-
-  inline UChar::UChar(const UCharReference &c) : uc(c.unicode()) { }
-
   /**
    * @short 8 bit char based string class
    */
@@ -158,12 +123,8 @@ namespace KJS {
 
     ~CString();
 
-    CString& append(const char* str, int len);
-    CString& append(const char* cstr);
-    CString& append(const CString& t) { return append(t.data, t.length); }
     CString &operator=(const char *c);
     CString &operator=(const CString &);
-    CString &operator+=(const CString &c) { return append(c); }
 
     size_t size() const { return length; }
     const char *c_str() const { return data; }
@@ -184,24 +145,24 @@ namespace KJS {
      */
     struct KJS_EXPORT Rep {
 
-      static Rep& create(UChar *d, int length);
-      static Rep& createCopying(const UChar *d, int length);
-      static Rep& create(Rep* base, int offset, int length);
-      static Rep& create(const RefPtr<Rep>& base, int offset, int length)
-					{ return create(base.get(), offset, length); }
+      static PassRefPtr<Rep> create(UChar *d, int l);
+      static PassRefPtr<Rep> createCopying(const UChar *d, int l);
+      static PassRefPtr<Rep> create(PassRefPtr<Rep> base, int offset, int length);
 
       void destroy();
-      
-      UChar *data() const { return offset + (baseString ? (baseString->buf + baseString->preCapacity) : (buf + preCapacity)); }
+
+      bool baseIsSelf() const { return baseString == this; }
+      UChar* data() const { return baseString->buf + baseString->preCapacity + offset; }
       int size() const { return len; }
-      
+
       unsigned hash() const { if (_hash == 0) _hash = computeHash(data(), len); return _hash; }
+      unsigned computedHash() const { assert(_hash); return _hash; } // fast path for Identifiers
       static unsigned computeHash(const UChar *, int length);
       static unsigned computeHash(const char* s, int length);
       static unsigned computeHash(const char *);
 
-      void ref() { ++rc; }
-      void deref() { if (--rc == 0) destroy(); }
+      Rep* ref() {  ++rc; return this; }
+      ALWAYS_INLINE void deref() { if (--rc == 0) destroy(); }
 
       // unshared data
       int offset;
@@ -209,7 +170,8 @@ namespace KJS {
       int rc;
       mutable unsigned _hash;
       bool isIdentifier;
-      UString::Rep *baseString;
+      UString::Rep* baseString;
+      size_t reportedCost;
 
       // potentially shared data
       UChar *buf;
@@ -217,7 +179,7 @@ namespace KJS {
       int capacity;
       int usedPreCapacity;
       int preCapacity;
-      
+
       static Rep null;
       static Rep empty;
     };
@@ -240,7 +202,7 @@ namespace KJS {
      * Constructs a string from a classical zero determined char string.
      */
     UString(const char *c);
-    UString(const char* c, int length);
+    UString(const char* c, size_t length);
     /**
      * Constructs a string from an array of Unicode characters of the specified
      * length.
@@ -257,6 +219,9 @@ namespace KJS {
      * Copy constructor. Makes a shallow copy only.
      */
     UString(const UString &s) : m_rep(s.m_rep) {}
+
+    UString(const Vector<UChar>& buffer);
+
     /**
      * Convenience declaration only ! You'll be on your own to write the
      * implementation for a construction from QString.
@@ -272,7 +237,7 @@ namespace KJS {
     /**
      * Convenience declaration only ! See UString(const QString&).
      */
-    KJS_EXTERNAL_EXPORT UString(const DOM::AtomicString&);
+    KJS_EXTERNAL_EXPORT UString(const khtml::AtomicString&);
 
     /**
      * Concatenation constructor. Makes operator+ more efficient.
@@ -299,6 +264,9 @@ namespace KJS {
      * Constructs a string from a double.
      */
     static UString from(double d);
+    
+    
+    static bool equal(const UString::Rep* a, const UString::Rep* b);
 
     struct Range {
     public:
@@ -315,7 +283,6 @@ namespace KJS {
      */
     UString& append(const UString& subStr, int subPos, int subLength = -1);
     UString& append(const UString& t);
-    UString& append(const UChar* t, int tSize);
     UString& append(const char* t);
     UString& append(const char* t, int tSize);
     UString& append(unsigned short);
@@ -395,12 +362,7 @@ namespace KJS {
     /**
      * Const character at specified position.
      */
-    UChar operator[](int pos) const;
-    /**
-     * Writable reference to character at specified position.
-     */
-    UCharReference operator[](int pos);
-
+    const UChar operator[](int pos) const;
     /**
      * Attempts an conversion to a number. Apart from floating point numbers,
      * the algorithm will recognize hexadecimal representations (as
@@ -416,9 +378,15 @@ namespace KJS {
     /**
      * Attempts an conversion to a 32-bit integer. ok will be set
      * according to the success.
-     * @param tolerateEmptyString if false, toUInt32 will return false for *ok for an empty string.
      */
     uint32_t toUInt32(bool *ok = 0) const;
+    /**
+     * Attempts an conversion to a 32-bit integer. ok will be set
+     * according to the success.
+     *
+     * @param tolerateEmptyString if false, toUInt32 will return false
+     * for *ok for an empty string.
+     */
     uint32_t toUInt32(bool *ok, bool tolerateEmptyString) const;
     uint32_t toStrictUInt32(bool *ok = 0) const;
 
@@ -451,21 +419,14 @@ namespace KJS {
      * Static instance of a null string.
      */
     static const UString &null();
-#ifdef KJS_DEBUG_MEM
-    /**
-     * Clear statically allocated resources.
-     */
-    static void globalClear();
-#endif
-
-    Rep *rep() const { return m_rep.get(); }
-    UString(PassRefPtr<Rep> r) : m_rep(r) { }
 
+    Rep* rep() const { return m_rep.get(); }
+    UString(PassRefPtr<Rep> r) : m_rep(r) { assert(m_rep); }
     void copyForWriting();
 
+    size_t cost() const;
   private:
-    UString(Rep& r) : m_rep(r) {}
-    static int expandedSize(int size, int otherSize);
+    size_t expandedSize(size_t size, size_t otherSize) const;
     int usedCapacity() const;
     int usedPreCapacity() const;
     void expandCapacity(int requiredLength);
@@ -479,7 +440,7 @@ namespace KJS {
     return (c1.uc == c2.uc);
   }
   KJS_EXPORT bool operator==(const UString& s1, const UString& s2);
-  KJS_EXPORT  inline bool operator!=(const UString& s1, const UString& s2) {
+  KJS_EXPORT inline bool operator!=(const UString& s1, const UString& s2) {
     return !KJS::operator==(s1, s2);
   }
   KJS_EXPORT bool operator<(const UString& s1, const UString& s2);
@@ -497,7 +458,7 @@ namespace KJS {
   KJS_EXPORT inline UString operator+(const UString& s1, const UString& s2) {
     return UString(s1, s2);
   }
-  
+
   KJS_EXPORT int compare(const UString &, const UString &);
 
   // Given a first byte, gives the length of the UTF-8 sequence it begins.
@@ -525,6 +486,45 @@ inline unsigned UString::toArrayIndex(bool *ok) const
     return i;
 }
 
+// We'd rather not do shared substring append for small strings, since
+// this runs too much risk of a tiny initial string holding down a
+// huge buffer.
+// FIXME: this should be size_t but that would cause warnings until we
+// fix UString sizes to be size_t instead of int
+static const int minShareSize = Collector::minExtraCostSize / sizeof(UChar);
+
+inline size_t UString::cost() const
+{
+   size_t capacity = (m_rep->baseString->capacity + m_rep->baseString->preCapacity) * sizeof(UChar);
+   size_t reportedCost = m_rep->baseString->reportedCost;
+   ASSERT(capacity >= reportedCost);
+
+   size_t capacityDelta = capacity - reportedCost;
+
+   if (capacityDelta < static_cast<size_t>(minShareSize))
+       return 0;
+
+   m_rep->baseString->reportedCost = capacity;
+   return capacityDelta;
+}
+
 } // namespace
 
+namespace WTF {
+
+    template<typename T> struct DefaultHash;
+    template<typename T> struct StrHash;
+
+    template<> struct StrHash<KJS::UString::Rep *> {
+        static unsigned hash(const KJS::UString::Rep *key) { return key->hash(); }
+        static bool equal(const KJS::UString::Rep *a, const KJS::UString::Rep *b) { return KJS::UString::equal(a, b); }
+        static const bool safeToCompareToEmptyOrDeleted = false;
+    };
+
+    template<> struct DefaultHash<KJS::UString::Rep *> {
+        typedef StrHash<KJS::UString::Rep *> Hash;
+    };
+} // namespace WTF
+
+
 #endif
