@@ -20,23 +20,25 @@
 #ifndef KROSS_ACTION_H
 #define KROSS_ACTION_H
 
-
 #include <QtCore/QVariant>
 #include <QtCore/QObject>
 #include <QtCore/QDir>
-#include <QtXml/QDomAttr>
-#include <QtGui/QAction>
 #include <QtCore/QUrl>
+#include <QtGui/QAction>
+#include <QtXml/QDomAttr>
+#include <QtScript/QScriptable>
 
 #include "errorinterface.h"
 #include "childreninterface.h"
 
 namespace Kross {
 
+    class Script;
+
     /**
      * The Action class is an abstract container to deal with scripts
-     * like a single standalone scriptfile. Each action holds a reference
-     * to by the matching @a Kross::Interpreter created @a Kross::Script
+     * like a single standalone script file. Each action holds a reference
+     * to the matching @a Kross::Interpreter created @a Kross::Script
      * instance.
      *
      * The \a Manager takes care of handling the \a Action instances
@@ -44,10 +46,10 @@ namespace Kross {
      * for those \a Action instances.
      *
      * Once you've such a Action instance you're able to perform actions
-     * with it like to execute scripting code.
+     * with it like executing scripting code.
      *
      * Following sample shows "Hello World." executed with the python
-     * interpreter;
+     * interpreter:
      * \code
      * # Create a new Kross::Action instance.
      * Kross::Action* action = new Kross::Action(0,"MyFirstScript");
@@ -60,7 +62,7 @@ namespace Kross {
      * \endcode
      *
      * Following sample demonstrates how to execute an external python script
-     * file. The script file itself is named "mytest.py" and contains;
+     * file. The script file itself is named "mytest.py" and contains:
      * \code
      * # this function got called from within C++
      * def myfunction(args):
@@ -73,7 +75,7 @@ namespace Kross {
      * MySecondQObject.someProperty = "Other string"
      * \endcode
      * Then you are able to load the script file, publish QObject instances
-     * and let the script do whatever it likes to do;
+     * and let the script do whatever it likes to do:
      * \code
      * # Publish a QObject instance for all Kross::Action instances.
      * Kross::Manager::self().addObject(myqobject1, "MyFirstQObject")
@@ -89,7 +91,11 @@ namespace Kross {
      * QVariant result = action->callFunction("myfunction", QVariantList()<<"Arg");
      * \endcode
      */
-    class KROSSCORE_EXPORT Action : public QAction, public ChildrenInterface, public ErrorInterface
+    class KROSSCORE_EXPORT Action
+        : public QAction
+        , public QScriptable
+        , public ChildrenInterface
+        , public ErrorInterface
     {
             Q_OBJECT
 
@@ -98,7 +104,7 @@ namespace Kross {
            /**
              * Constructor.
              *
-             * \param parent The parent QObject this \a Action is child of.
+             * \param parent The parent QObject this \a Action is a child of.
              * \param name The unique name this Action has. It's used
              * e.g. at the \a Manager to identify the Action. The
              * name is accessible via \a QObject::objectName .
@@ -108,11 +114,11 @@ namespace Kross {
             /**
              * Constructor.
              *
-             * \param parent The parent QObject this \a Action is child of.
-             * \param url The URL should point  to a valid scriptingfile.
+             * \param parent The parent QObject this \a Action is a child of.
+             * \param url The URL should point to a valid scripting file.
              * This \a Action will be filled with the content of the
-             * file (e.g. the file is readed and \a code should return
-             * it's content and it's also tried to determinate the
+             * file (e.g. the file is read and \a code should return
+             * its content and it's also tried to determine the
              * \a interpreter ). Remember to use QUrl c'tor explicitly.
              * The name will be set to url.path()
              */
@@ -137,63 +143,44 @@ namespace Kross {
              */
             QDomElement toDomElement() const;
 
-            /**
-             * \return a map of options this \a Action defines.
-             * The options are returned call-by-ref, so you are able to
-             * manipulate them.
-             */
-            QMap<QString, QVariant>& options() const;
-
-            /**
-             * \return the value of the option defined with \p name .
-             * If there doesn't exists an option with such a name,
-             * the \p defaultvalue is returned.
-             */
-            QVariant option(const QString& name, const QVariant& defaultvalue = QVariant());
-
-            /**
-             * Set the \a Interpreter::Option value.
-             */
-            bool setOption(const QString& name, const QVariant& value);
-
-            /**
-             * \return the list of functionnames.
-             */
-            QStringList functionNames();
-
-            /**
-             * Call a function in the script.
-             *
-             * \param name The name of the function which should be called.
-             * \param args The optional list of arguments.
-             */
-            QVariant callFunction(const QString& name, const QVariantList& args = QVariantList());
-
             /**
              * Initialize the \a Script instance.
              *
-             * Normaly it's not needed to call this function direct cause
-             * if will be internaly called if needed (e.g. on \a execute ).
+             * Normally there is no need to call this function directly because
+             * if will be called internally if needed (e.g. on \a execute ).
              *
-             * \return true if the initialization was successfully else
+             * \return true if the initialization was successful else
              * false is returned.
              */
             bool initialize();
 
             /**
-             * Finalize the \a Script instance and free's any cached or still
-             * running executions. Normaly it's not needed to call this
-             * function direct cause the \a Action will take care
+             * Finalize the \a Script instance and frees any cached or still
+             * running executions. Normally there is no need to call this
+             * function directly because the \a Action will take care
              * of calling it if needed.
              */
             void finalize();
 
             /**
-             * \return true if the action is finalized what means the
+             * \return true if the action is finalized, which means the
              * action is currently not running.
              */
             bool isFinalized() const;
 
+            /**
+             * \return the \a Kross::Script implementation used by the scripting
+             * backend. This returns NULL until the action got triggered or if
+             * there was a error before that.
+             *
+             * Normaly it shouldn't be necessary to deal with the scripting backend
+             * depending instance of a \a Kross::Script implementation since this
+             * \a Action class already decorates all the things needed. It
+             * may however be useful to provide additional interpreter dependent
+             * functionality.
+             */
+            Script* script() const;
+
         public Q_SLOTS:
 
             /**
@@ -238,12 +225,12 @@ namespace Kross {
             void setEnabled(bool enabled);
 
             /**
-             * \return the scriptfile that should be executed.
+             * \return the script file that should be executed.
              */
             QString file() const;
 
             /**
-             * Set the scriptfile that should be executed.
+             * Set the script file that should be executed.
              */
             bool setFile(const QString& scriptfile);
 
@@ -274,16 +261,67 @@ namespace Kross {
              */
             QString currentPath() const;
 
+            /**
+             * Add a QObject instance to the action. This instance will
+             * be published to scripts.
+             */
+            void addQObject(QObject* obj, const QString &name = QString());
+
+            /**
+             * \return the QObject with the object name \p name .
+             */
+            QObject* qobject(const QString &name) const;
+
+            /**
+             * \return a list of QObject object names.
+             */
+            QStringList qobjectNames() const;
+
+            /**
+             * \return a map of options this \a Action defines.
+             * The options are returned call-by-ref, so you are able to
+             * manipulate them.
+             */
+            QVariantMap options() const;
+
+            /**
+             * \return the value of the option defined with \p name .
+             * If there doesn't exist an option with such a name,
+             * the \p defaultvalue is returned.
+             */
+            QVariant option(const QString& name, const QVariant& defaultvalue = QVariant());
+
+            /**
+             * Set the \a Interpreter::Option value.
+             */
+            bool setOption(const QString& name, const QVariant& value);
+
+            /**
+             * \return the list of functionnames.
+             */
+            QStringList functionNames();
+
+            /**
+             * Call a function in the script.
+             *
+             * \param name The name of the function which should be called.
+             * \param args The optional list of arguments.
+             */
+            QVariant callFunction(const QString& name, const QVariantList& args = QVariantList());
+
         Q_SIGNALS:
 
             /**
              * This signal is emitted if the content of the Action
              * was changed. The \a ActionCollection instances this Action
-             * is child of are connected with this signal to fire up
-             * there own updated signal if an Action of them was updated.
+             * is a child of are connected with this signal to fire up
+             * their own updated signal if an Action of them was updated.
              */
             void updated();
 
+            /// This signal is emitted when the data of the Action is changed
+            void dataChanged(Action*);
+
             /**
              * This signal is emitted before the script got executed.
              */
@@ -317,5 +355,7 @@ namespace Kross {
 
 }
 
+Q_DECLARE_METATYPE(Kross::Action*)
+
 #endif
 
